/*
 * anerouge.c
 *
 *  Created on: 11 avr. 2009
 *      Author: Nahuel Fouilleul
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utils.h"
#include "puzzle.h"
#include "anerouge.h"

int DEBUG = 0;

char klotski[K_LINES][K_COLS1];
byte positions[12], positionsSav[12];
byte posOpt[8];
int mask, maskSav;
byte cas = 1;

char * getPositionsString() {
	static char stringPositions[100];
	sprintf(stringPositions,
			"[%d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d]\n", positions[0],
			positions[1], positions[2], positions[3], positions[4],
			positions[5], positions[6], positions[7], positions[8],
			positions[9], positions[10], positions[11]);
	return stringPositions;
}

char * getPosOptString() {
	static char stringPosOpt[100];
	sprintf(stringPosOpt, "[%d, %d, %d, %d, %d, %d, %d, %d]\n", posOpt[0],
			posOpt[1], posOpt[2], posOpt[3], posOpt[4], posOpt[5], posOpt[6],
			posOpt[7]);
	return stringPosOpt;
}

// voir commentaire : cas 0 existe pas, cas 1 C(12,5)*C(11,0), etc.
int MAX_CONF_CAS[][2] = { { 0, 0 }, { 792, 1 }, { 210, 11 }, { 56, 55 }, { 28,
		165 }, { 6, 330 }, { 1, 462 } };
byte CONF_CAS[][4] = { { 0, 0, 0, 0 }, { 12, 5, 11, 0 }, { 10, 4, 11, 1 }, { 8,
		3, 11, 2 }, { 8, 2, 11, 3 }, { 6, 1, 11, 4 }, { 12, 0, 11, 5 } };

char * getFileName() {
	static char filename[32];
	sprintf(filename, "anerouge%d.bin", cas);
	return filename;
}

const Move moves[] = { { 0, M_SQ, -K_COLS1, 1 }, { 1, M_SQ, K_COLS1, 1 }, { 2,
		M_SQ, -1, 1 }, { 3, M_SQ, 1, 1 }, { 4, M_VR, -K_COLS1, 1 }, { 5, M_VR,
		K_COLS1, 1 }, { 6, M_VR, -1, 1 }, { 7, M_VR, 1, 1 }, { 8, M_HR,
		-K_COLS1, 1 }, { 9, M_HR, K_COLS1, 1 }, { 10, M_HR, -1, 1 }, { 11,
		M_HR, 1, 1 }, { 12, M_AR, -K_COLS1, 1 }, { 13, M_AR, K_COLS1, 1 }, {
		14, M_AR, -1, 1 }, { 15, M_AR, 1, 1 }, { 16, M_SQ, -K_COLS1, 0 }, { 17,
		M_SQ, K_COLS1, 0 }, { 18, M_SQ, -1, 0 }, { 19, M_SQ, 1, 0 }, { 20,
		M_VR, -K_COLS1, 0 }, { 21, M_VR, K_COLS1, 0 }, { 22, M_VR, -1, 0 }, {
		23, M_VR, 1, 0 }, { 24, M_HR, -K_COLS1, 0 }, { 25, M_HR, K_COLS1, 0 },
		{ 26, M_HR, -1, 0 }, { 26, M_HR, 1, 0 } };

void goBack() {
	int i;
	for (i = 0; i < 12; i++) {
		positions[i] = positionsSav[i];
	}
	mask = maskSav;
	setFromPositions();
}

int testMove(byte p, int f, byte c) {
	int m = mask;
	m ^= (f << p);
	return c >= 0 && (m & f << c) == 0 && c % K_COLS1 < K_COLS
			- ((f & 2) == 2 ? 1 : 0) && c < 24 - ((f & 32) == 32 ? K_COLS1 : 0);
}

void doMove(byte p, int f, byte k, byte c) {
	mask ^= f << p;
	mask |= f << c;
	positions[k] = c;
}

int exec(Move m) {
	byte p, k, n = 0, c, dir = m.dir;
	int f = m.forme;
	int i, r = 0;
	//memncpy(positions, 0, positionsSav, 0, 12);
	for (i = 0; i < 12; i++) {
		positionsSav[i] = positions[i];
	}
	maskSav = mask;
	switch (f) {
	case M_SQ:
		for (k = 2; k < 6; k++) {
			p = positions[k];
			c = (byte) (p + dir);
			if (testMove(p, f, c) && (m.first || n++ > 0)) {
				doMove(p, f, k, c);
				if (positions[0] == c) {
					positions[0] = p;
				} else {
					positions[1] = p;
				}
				r = true;
				break;
			}
		}
		break;
		// vertical 33
	case M_VR:
		for (k = 6; k < 12 - cas; k++) {
			c = p = positions[k];
			c += dir;
			if (testMove(p, f, c) && (m.first || n++ > 0)) {
				doMove(p, f, k, c);
				if (dir == 1 || dir == -1) {
					positions[0] = p;
					positions[1] = (byte) (p + K_COLS1);
				} else if (dir == -K_COLS1) {
					if (positions[0] == c) {
						positions[0] = (byte) (p - dir);
					} else {
						positions[1] = (byte) (p - dir);
					}
				} else if (positions[0] == c + dir) {
					positions[0] = p;
				} else {
					positions[1] = p;
				}
				r = true;
				break;
			}
		}
		break;
		// horizontal 3
	case M_HR:
		for (k = (byte) (12 - cas); k < 11; k++) {
			c = p = positions[k];
			byte c2 = c += dir;
			c2 += dir;
			if (testMove(p, f, c) && (m.first || n++ > 0)) {
				doMove(p, f, k, c);
				if (dir == K_COLS1 || dir == -K_COLS1) {
					positions[0] = p++;
					positions[1] = p;
				} else if (dir == -1) {
					if (positions[0] == c) {
						positions[0] = (byte) (p - dir);
					} else {
						positions[1] = (byte) (p - dir);
					}
				} else if (positions[0] == c + dir) {
					positions[0] = p;
				} else {
					positions[1] = p;
				}
				r = true;
				break;
			}
		}
		break;
		// ane rouge 99
	case M_AR:
		k = 11;
		c = p = positions[11];
		byte c2 = c += dir;
		c2 += dir;
		if (testMove(p, f, c)) {
			doMove(p, f, k, c);
			if (dir == K_COLS1) {
				positions[0] = p;
				positions[1] = (byte) (p + 1);
			} else if (dir == -K_COLS1) {
				positions[0] = (byte) (p - dir);
				positions[1] = (byte) (p - dir + 1);
			} else if (dir == 1) {
				positions[0] = p;
				positions[1] = (byte) (p + K_COLS1);
			} else {
				positions[0] = (byte) (p - dir);
				positions[1] = (byte) (p - dir + K_COLS1);
			}
			r = true;
		}
		break;
	}
	if (r) {
		if (DEBUG) {
			puts(getPositionsString());
			puts(getMoveString(m));
		}
		setFromPositions();
	}
	return r;
}

Move inv(Move m) {
	return moves[m.id ^ 1];
}

char * getMoveString(Move m) {
	static char movestring[4];
	movestring[3] = 0;
	byte b = 0;
	if (m.first) {
		movestring[b++] = ' ';
	} else {
		movestring[b++] = '2'; //²';
	}
	switch (m.forme) {
	case 1:
		movestring[b++] = '.';
		break;
	case 33:
		movestring[b++] = '|';
		break;
	case 3:
		movestring[b++] = '-';
		break;
	case 99:
		movestring[b++] = '#';
		break;
	}
	switch (m.dir) {
	case -1:
		movestring[b++] = '<'; //←';
		break;
	case 1:
		movestring[b++] = '>'; //→';
		break;
	case -K_COLS1:
		movestring[b++] = '^'; //↑';
		break;
	case K_COLS1:
		movestring[b++] = 'v'; //↓';
		break;
	}
	return movestring;
}

Move * getMoves() {
	return (Move *) moves;
}

int getSize() {
	return 12 * MAX_CONF_CAS[cas][0] * MAX_CONF_CAS[cas][1] * 15;
}

int setFromInt(int i0) {
	int a, b;
	for (a = 0; a < K_LINES; a++) {
		for (b = 0; b < K_COLS; b++) {
			klotski[a][b] = CUNDEF;
		}
	}
	for (a = 0; a < 12; a++) {
		positions[a] = CUNDEF;
	}
	for (a = 0; a < 12; a++) {
		posOpt[a] = 0;
	}
	int i = i0;
	itob((byte) 6, (byte) 2, i % 15, (byte) 5, posOpt);
	i /= 15;
	// V en premier
	itob(CONF_CAS[cas][0], CONF_CAS[cas][1], i % MAX_CONF_CAS[cas][0],
			(byte) (cas - 1), posOpt);
	i /= MAX_CONF_CAS[cas][0];
	itob(CONF_CAS[cas][2], CONF_CAS[cas][3], i % MAX_CONF_CAS[cas][1],
			(byte) 0, posOpt);
	i /= MAX_CONF_CAS[cas][1];
	posOpt[7] = (byte) i;
	if (DEBUG) {
		printf("fromInt:%s\n", getPosOptString());
	}
	return setFromPosOpt(posOpt);
}

int setFromPosOpt(byte * posOpt) {
	mask = MASK;
	byte i, j, k = 0;
	i = positions[11] = code(posOpt[7], C_AR, K_COLS1);
	mask ^= M_AR << i;
	j = codej(i, K_COLS1);
	i = codei(i, K_COLS1);
	klotski[i][j] = klotski[i + 1][j] = klotski[i][j + 1] = klotski[i + 1][j
			+ 1] = 'L';
	// horizontal rectangles
	i = 10;
	j = -1;
	while (k < cas - 1) {
		do {
			j++;
			if (j % K_COLS1 < K_COLS - 1 && (M_HR & mask >> j) == M_HR) {
				posOpt[k]--;
			} else if (j >= 22) {
				return false;
			}
		} while (posOpt[k] >= 0);
		positions[i--] = j;
		mask ^= M_HR << j;
		klotski[codei(j, K_COLS1)][codej(j, K_COLS1)] = klotski[codei(j,
				K_COLS1)][codej(j, K_COLS1) + 1] = (char) ('G' + k++);
	}
	// vertical rectangles
	i = 6;
	j = -1;
	while (k < 5) {
		do {
			j++;
			if (j % K_COLS1 < K_COLS && (M_VR & mask >> j) == M_VR) {
				posOpt[k]--;
			} else if (j >= 18) {
				return false;
			}
		} while (posOpt[k] >= 0);
		positions[i++] = j;
		mask ^= M_VR << j;
		klotski[codei(j, K_COLS1)][codej(j, K_COLS1)] = klotski[codei(j,
				K_COLS1) + 1][codej(j, K_COLS1)] = (char) ('G' + k++);
	}
	// square pos
	i = 0;
	j = -1;
	byte r = 0;
	while (k < 7) {
		do {
			j++;
			if (j % K_COLS1 < K_COLS && (1 & mask >> j) == 1) {
				if (posOpt[k]-- > 0) {
					positions[2 + r] = j;
					klotski[codei(j, K_COLS1)][codej(j, K_COLS1)] = (char) ('C'
							+ r++);
					mask ^= 1 << j;
				}
			} else if (j >= 23) {
				return false;
			}
		} while (posOpt[k] >= 0);
		positions[i++] = j;
		klotski[codei(j, K_COLS1)][codej(j, K_COLS1)] = (char) ('A' + k++ - 5);
	}
	while (r < 4) {
		j++;
		if (j % K_COLS1 < K_COLS && (1 & mask >> j) == 1) {
			positions[2 + r] = j;
			klotski[codei(j, K_COLS1)][codej(j, K_COLS1)] = (char) ('C' + r++);
			mask ^= (1 << j);
		} else if (j >= 23) {
			return false;
		}
	}
	mask ^= MASK;
	return true;
}

int setFromPositions() {
	byte i, j, k, r = positions[11];
	int a, b;
	for (a = 0; a < K_LINES; a++) {
		for (b = 0; b < K_COLS; b++) {
			klotski[a][b] = CUNDEF;
		}
	}
	i = codei(r, K_COLS1);
	j = codej(r, K_COLS1);
	klotski[i][j] = klotski[i + 1][j] = klotski[i][j + 1] = klotski[i + 1][j
			+ 1] = 'L';
	// horizontal rectangles
	i = 10;
	for (k = 0; k < cas - 1; k++) {
		r = positions[i--];
		if (DEBUG && (klotski[codei(r, K_COLS1)][codej(r, K_COLS1)] != CUNDEF
				|| klotski[codei(r, K_COLS1)][codej(r, K_COLS1) + 1] != CUNDEF)) {
			puts("setFromPos HR");
			return false;
		}
		klotski[codei(r, K_COLS1)][codej(r, K_COLS1)] = klotski[codei(r,
				K_COLS1)][codej(r, K_COLS1) + 1] = (char) ('K' - k);
	}
	i = 6;
	// vertical rectangles
	for (; k < 5; k++) {
		r = positions[i++];
		if (DEBUG && (klotski[codei(r, K_COLS1)][codej(r, K_COLS1)] != CUNDEF
				|| klotski[codei(r, K_COLS1) + 1][codej(r, K_COLS1)] != CUNDEF)) {
			puts("setFromPos VR");
			return false;
		}
		klotski[codei(r, K_COLS1)][codej(r, K_COLS1)] = klotski[codei(r,
				K_COLS1) + 1][codej(r, K_COLS1)] = (char) ('K' - k);
	}
	// square pos
	i = 0;
	for (; k < 7; k++) {
		r = positions[i++];
		if (DEBUG && klotski[codei(r, K_COLS1)][codej(r, K_COLS1)] != CUNDEF) {
			puts("setFromPos SQE");
			return false;
		}
		klotski[codei(r, K_COLS1)][codej(r, K_COLS1)] = (char) ('A' + k - 5);
	}
	k = 0;
	for (i = 0; i < K_LINES && k < 4; i++) {
		for (j = 0; j < K_COLS && k < 4; j++) {
			if (klotski[i][j] == CUNDEF) {
				klotski[i][j] = (char) ('C' + k++);
			}
		}
	}
	return k == 4;
}

int setFromString(char * s) {
	int i, j = 0, k;
	for (i = 0; s[i] != 0 && j < K_LINES; i++) {
		if (s[i] == '\n') {
			j++;
		}
	}
	if (s[i] != '\0' && s[i] != '\n') {
		puts(
				"AneRouge.setFromString: Mauvais nombre de lignes doit etre " STR(K_LINES));
		return -1;
	}
	for (i = 0; i < K_LINES; i++) {
		k = 0;
		for (j = K_LINES * i; s[j] != '\n' && s[j] != 0; j++, k++)
			;
		if (k != K_COLS) {
			printf(
					"AneRouge.setFromString ligne %d: Mauvais nombre de caracteres sur une ligne doit etre " STR(K_COLS) "\n",
					i);
			return -1;
		}
		for (j = K_LINES * i; s[j] != '\n' && s[j] != 0; j++) {
			char c = s[j];
			if (c < 'A' || c > 'L') {
				printf(
						"AneRouge.setFromString ligne %d, colonne %d: Mauvaise lettre d'identification doit etre entre A et L : %c\n",
						i, j % 5, c);
				return -1;
			}
			klotski[i][j % 5] = c;
		}
	}
	return toInt();
}

int toInt() {
	int i, j = 0, k;
	byte p = 0;
	byte namPos[12];
	cas = 1;
	for (i = 0; i < 12; i++) {
		namPos[i] = positions[i] = UNDEF;
	}
	for (i = 0; i < K_LINES; i++) {
		for (j = 0; j < K_COLS1; j++, p++) {
			switch (klotski[i][j]) {
			case 'A':
			case 'B':
				for (k = 0; k < 2 && positions[k] != UNDEF; k++)
					;
				if (k == 2) {
					puts("Erreur: plus de deux cases vides A ou B");
					return -1;
				}
				positions[k] = p;
				if (namPos[klotski[i][j] - 'A'] == UNDEF) {
					namPos[klotski[i][j] - 'A'] = (byte) k;
				} else {
					puts("Erreur: doublon nom cases vides A ou B");
					return -1;
				}
				break;
			case 'C':
			case 'D':
			case 'E':
			case 'F':
				for (k = 2; k < 6 && positions[k] != UNDEF; k++)
					;
				if (k == 6) {
					puts("Erreur: plus de quatre carres C, D, E ou F");
					return -1;
				}
				positions[k] = p;
				if (namPos[klotski[i][j] - 'A'] == UNDEF) {
					namPos[klotski[i][j] - 'A'] = (byte) k;
				} else {
					puts("Erreur: doublon nom quatre carres C, D, E ou F");
					return -1;
				}
				break;
			case 'G':
			case 'H':
			case 'I':
			case 'J':
			case 'K':
				if (i < 4 && klotski[i + 1][j] == klotski[i][j]) {
					klotski[i + 1][j] = ' ';
					for (k = 6; k < 11 && positions[k] != UNDEF; k++)
						;
				} else if (j < 3 && klotski[i][j + 1] == klotski[i][j]) {
					klotski[i][j + 1] = ' ';
					// horizontal
					cas++;
					for (k = 10; k > 5 && positions[k] != UNDEF; k--)
						;
				} else {
					puts("Erreur: position d'un des rectangles G, H, I, J ou K");
					puts(klotski[i]);
					puts(getPuzzleString());
					return -1;
				}
				if (k == 5 || k == 11) {
					puts("Erreur: plus de cinq rectangles G, H, I, J ou K");
					return -1;
				}
				positions[k] = p;
				if (namPos[klotski[i][j] - 'A'] == UNDEF) {
					namPos[klotski[i][j] - 'A'] = (byte) k;
				} else {
					puts("Erreur: doublon nom rectangles G, H, I, J ou K");
					return -1;
				}
				break;
			case 'L':
				if (positions[11] != UNDEF) {
					puts("Erreur: plus d'un ane rouge L");
					return -1;
				}
				if (i == 4 || j == 3) {
					puts("Erreur: position de l'ane rouge L");
					return -1;
				}
				if (klotski[i + 1][j] == 'L' && klotski[i + 1][j + 1] == 'L'
						&& klotski[i][j + 1] == 'L') {
					positions[11] = p;
				} else {
					puts("Erreur: L doit etre carre 2x2");
					return -1;
				}
				klotski[i + 1][j] = ' ';
				klotski[i + 1][j + 1] = ' ';
				klotski[i][j + 1] = ' ';
				if (namPos[klotski[i][j] - 'A'] == UNDEF) {
					namPos[klotski[i][j] - 'A'] = (byte) 11;
				} else {
					puts("Erreur: doublon de l'ane rouge L");
					return -1;
				}
				break;
			default:
				break;
			}
		}
	}
	int r = posToInt();
	if (DEBUG) {
		printf("toInt:%s -> %d\n", getPositionsString(), r);
	}
	return r;
}

int posToInt() {
	int i = 11, j = 0, k, m, r;
	byte p = 0;
	mask = m = MASK;
	p = posOpt[7] = positions[i--];
	klotski[codei(p, K_COLS1) + 1][codej(p, K_COLS1)] = klotski[codei(p,
			K_COLS1) + 1][codej(p, K_COLS1) + 1]
			= klotski[codei(p, K_COLS1)][codej(p, K_COLS1) + 1]
					= klotski[codei(p, K_COLS1)][codej(p, K_COLS1)];
	m ^= M_AR << p;
	r = code(p, K_COLS1, C_AR);
	for (j = 0; j < 7; j++) {
		posOpt[j] = 0;
	}
	// horizontal rectangles
	j = 0;
	for (k = 0; k < cas - 1; k++) {
		p = positions[i--];
		klotski[codei(p, K_COLS1)][codej(p, K_COLS1) + 1] = klotski[codei(p,
				K_COLS1)][codej(p, K_COLS1)];
		for (; j < p; j++) {
			if (j % K_COLS1 < K_COLS - 1 && (M_HR & m >> j) == M_HR) {
				posOpt[k]++;
			}
		}
		m ^= (M_HR << p);
	}
	// vertical rectangles
	i = 6;
	j = 0;
	for (; k < 5; k++) {
		p = positions[i++];
		klotski[codei(p, K_COLS1) + 1][codej(p, K_COLS1)] = klotski[codei(p,
				K_COLS1)][codej(p, K_COLS1)];
		for (; j < p; j++) {
			if ((j % K_COLS1 < K_COLS) && (M_VR & m >> j) == M_VR) {
				posOpt[k]++;
			}
		}
		m ^= (M_VR << p);
	}
	// H en premier
	r *= MAX_CONF_CAS[cas][1];
	r += btoi(CONF_CAS[cas][2], CONF_CAS[cas][3], (byte) 0, posOpt);
	r *= MAX_CONF_CAS[cas][0];
	r += btoi(CONF_CAS[cas][0], CONF_CAS[cas][1], (byte) (cas - 1), posOpt);
	// square pos
	i = 0;
	j = 0;
	for (; k < 7; k++) {
		p = positions[i++];
		for (; j < p; j++) {
			if ((j % K_COLS1 < K_COLS) && (1 & m >> j) == 1) {
				posOpt[k]++;
			}
		}
		int t = 1 << p;
		m ^= t;
		mask ^= t;
	}
	r *= 15;
	r += btoi((byte) 6, (byte) 2, (byte) 5, posOpt);
	if (DEBUG) {
		puts(getPosOptString());
	}
	return r;
}

char * getPuzzleString() {
	static char klotskiString[32];
	int i, j, t = 0;
	for (i = 0; i < K_LINES; i++) {
		for (j = 0; j < K_COLS; j++) {
			klotskiString[t++] = klotski[i][j];
		}
		klotskiString[t++] = '\n';
	}
	klotskiString[t++] = '\0';
	return klotskiString;
}

void testBin() {
	byte a[] = { 5, 1, 0 };
	int i;
	puts("binomial(5,2)");
	printf("%d\n", binomial(5, 2));
	printf("%d %d %d\n", a[0], a[1], a[2]);
	printf("%d\n", i = btoi((byte) 11, (byte) 3, (byte) 0, (byte *) a));
	printf("%d %d %d\n", a[0], a[1], a[2]);
	for (i = 0; i < 120; i++) {
		itob((byte) 10, (byte) 3, i, (byte) 0, (byte *) a);
		printf("%d %d %d\n", a[0], a[1], a[2]);
	}
}

void testAr() {
	setFromString("GHKK\nGHCI\nDLLI\nELLJ\nFABJ\n");
	puts(getPuzzleString());
	printf("%d, %d\n", toInt(), setFromInt(toInt()));
	puts(getPuzzleString());
	printf("%d, %d\n", 374637, setFromInt(374637));
	puts(getPuzzleString());
}

void testArseq() {
	int deb = getSize() / 12;
	int max = 11* deb ;
	int c = 0, i;
	cas = 2;
	deb = 10* deb ;
	for (i = deb; i < max; i++) {
		if (setFromInt(i)) {
			c++;
			printf("### %d ###\n", i);
			puts(getPuzzleString());
		}
	}
	printf("%d : %d / %d\n", cas, c, max - deb);
}

void testSolve(int id, int print) {
	//setFromString("GLLH\nGLLH\nIKKJ\nICDJ\nEABF");
	//setFromString("GLLH\nGLLH\nICDJ\nIKKJ\nEABF");
	//setFromString("GLLH\nGLLH\nCIJD\nEIJF\nAKKB");
	//setFromString("GLLH\nGLLH\nIKKJ\nICDJ\nEABF");
	//setFromString("GLLH\nGLLH\nCIID\nJEFK\nJABK");
	//setFromString("CLLD\nGLLH\nGEFH\nJIIK\nJABK");
	//setFromString("GLLH\nGLLH\nCDEF\nIJJK\nIABK");
	//setFromString("GLLH\nGLLH\nCIID\nEJKF\nAJKB");
	//setFromString("GGHH\nILLJ\nILLJ\nCKKE\nDABF");
	//int id = toInt();
	if (load(NULL) != 1) {
		// Le fichier n'a pas été trouvé pour le cas courant
		// On genere l'arbre pour le cas courant :
		printf("Generation arbre - cas : %d\n", cas);
		// le nb majorant le nb de combinaison pour une position de l'ar
		// fixe
		int nbc = MAX_CONF_CAS[cas][0] * MAX_CONF_CAS[cas][1] * 15;
		// le but est d'avoir l'ar en pos 10
		// pour construire la liste des etats finaux
		int deb = nbc * 10, fin = nbc * 11;
		// 8250 est le nb maximum de combinaisons (atteint pour cas 3)
		int l[8251];
		int i = 0, j;
		for (j = deb; j < fin; j++) {
			if (setFromInt(j)) {
				// j valide
				l[i++] = j;
			}
		}
		// -1 pour indiquer la fin de la liste
		l[i++] = -1;
		if (i == 0) {
			puts("Erreur : pas d'etat final");
			exit(1);
		}
		load(l);
	}
	setFromInt(id);
	puts("==");
	puts("position depart : ");
	puts(getPuzzleString());
	printf("id position depart : %d\n", id);
	int i = getF(id);
	if (i >= 0) {
		printf("une solution optimale : en %d coups\n", i);
		puts(solveOne(print));
		puts("position finale : ");
		puts(getPuzzleString());
	} else {
		puts("problème impossible");
	}
	puts("==");
}

void usage(char * prgname) {
	printf("usage %s -h : affiche cette aide\n", prgname);
	printf("usage %s -v : affiche le copyright du programme\n", prgname);
	printf(
			"usage %s -f <fichier> : fichier décrivant la position initiale des pièces\n",
			prgname);
	puts("options:");
	puts("-p : affiche tous les etats d'une solution optimale.");
	puts("-s : crée un fichier pour optimiser le temps de génération");
	puts("-d : mode debug");
}

void copyr(char * prgname) {
	printf("%s : copyright 2009 Nahuel Fouilleul\n", prgname);
}

int main(int argc, char ** argv) {
	if (argc <= 0 || argc > 6) {
		puts("Erreur nombre d'arguments");
		usage(argv[0]);
		return 1;
	}
	int i, print = false;
	char * filename = NULL;
	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-h") == 0) {
			usage(argv[0]);
			return 0;
		} else if (strcmp(argv[i], "-v") == 0) {
			copyr(argv[0]);
			return 0;
		} else if (strcmp(argv[i], "-f") == 0) {
			i++;
			if (i >= argc) {
				usage(argv[0]);
				return 1;
			}
			filename = argv[i];
		} else if (strcmp(argv[i], "-p") == 0) {
			print = true;
		} else if (strcmp(argv[i], "-d") == 0) {
			DEBUG = true;
		} else if (strcmp(argv[i], "-s") == 0) {
			setPersist();
		} else {
			printf("%s : option inconnue\n", argv[i]);
			usage(argv[0]);
			exit(1);
		}
	}
	// initialisation
	for (i = 0; i < K_LINES; i++) {
		klotski[i][K_COLS] = CUNDEF;
	}
	if (filename == NULL) {
		usage(argv[0]);
		return 1;
	}
	FILE * filein = (FILE *) fopen(filename, "rb");
	if (filein == NULL) {
		printf("Erreur de lecture du fichier d'entrée : %s\n", filename);
		return 1;
	}
	char buf[40];
	buf[39] = '\0';
	i = fread(buf, 1, 39, filein);
	buf[i] = '\0';
	fclose(filein);
	i = setFromString(buf);
	if (i > -1) {
		testSolve(i, print);
	}
	return 0;
}
